const St           = imports.gi.St;
const Mainloop     = imports.mainloop;
const Main         = imports.ui.main;
const GLib         = imports.gi.GLib;
const Gio          = imports.gi.Gio;
const Lang         = imports.lang;
const Gtk          = imports.gi.Gtk;
const Shell        = imports.gi.Shell;
const PopupMenu    = imports.ui.popupMenu;
const PanelMenu    = imports.ui.panelMenu;
const MessageTray  = imports.ui.messageTray; 
const Gettext      = imports.gettext.domain('gnome-shell-extension-langswitcher');
const _            = Gettext.gettext;

// Import own libs:
const Me           = imports.misc.extensionUtils.getCurrentExtension();
const Time         = Me.imports.timer;
const Settings     = Me.imports.settings;

//fields
let _path          = null;
let _timer         = null;
let _layout_timer  = null;
let _text          = null;
let _indicator     = null;
let _codes_table   = null;
let _layout        = "";  

function LangSwitcherIndicator() 
{
    this._init.apply(this, arguments);
}

LangSwitcherIndicator.prototype = 
{
  __proto__: PanelMenu.Button.prototype,

  _init: function() 
  {
    PanelMenu.Button.prototype._init.call(this, St.Align.START);
    this.mainBox    = null;
    this.buttonText = new St.Label({text:_( get_layout() )});
    this.buttonText.set_style("text-align:center;");
    this.actor.add_actor(this.buttonText);
    this.buttonText.get_parent().add_style_class_name("panelButtonWidth");
    let _layouts = get_layouts();
    for (var i = 0; i < _layouts.length; i++)
    {
        this._myMenu = new PopupMenu.PopupMenuItem(_layouts[i]);
        this.menu.addMenuItem(this._myMenu);
        let _layout_number = i.toString();
        this._myMenu.connect('activate', Lang.bind(this, function()
	    {
 		    GLib.spawn_command_line_sync(_path + "/xkblayout-state set " + _layout_number, null);
	    }));
    }
  },
   
  _refresh: function() 
  {
    this.buttonText.set_text(_layout);
    this.actor.add_actor(this.buttonText);
    this.buttonText.get_parent().add_style_class_name("panelButtonWidth");
  }
};

function init(extensionMeta)
{   
	_path = extensionMeta.path;
  GLib.spawn_command_line_sync("chmod +755 " + _path + "/xkblayout-state",null);
  Gtk.IconTheme.get_default().append_search_path(_path + "/icons");
  _codes_table  = Settings.LANG_CODES.split(":");
}

function create_indicator()
{
    _timer        = new Time.Timer();
    _layout_timer = new Time.Timer();
    _timer.set_delay_time(500);
    _layout_timer.set_delay_time(Settings.SHOW_TIME);
    
    _indicator = new LangSwitcherIndicator();
    Main.panel.addToStatusArea('langswither', _indicator, Settings.NUMBER);

    _layout = get_layout();
    _timer.setCallback(function()
    {
        let _new_layout = get_layout();
        if ( _new_layout !=  _layout)
        {
           _layout = _new_layout;
	       _indicator._refresh();
	       if(Settings.SHOW_LAYOUT == true)
	       {
	       	    _showLayout() ;
	       }
	    }    
    });

    _layout_timer.setCallback(function()
    {
      _text.destroy();   
    });

    _timer.begin();
}

function destroy_indicator()
{
  _indicator.destroy();
  _layout_timer.stop();
  _timer.stop();
}

function enable() 
{
    create_indicator();
}

function disable()
{
	destroy_indicator();
}

function correct_case(_string)
{
    if(Settings.CASE == 0)
    {
       _string = _string.toUpperCase();
    } 
    else
    {
       if(Settings.CASE == 1)
       {
          _string = _string.toLowerCase();
       }
    }
    return _string; 
}

function get_layout()
{
    var _string = new String("unknown");
    var _command = GLib.spawn_command_line_sync(_path + "/xkblayout-state print \"%s\"",null);
    if (_command[0].toString() == "true")
    {
        _string = _command[1].toString();        
    }

    return correct_case(_string);
}

function get_layouts()
{
    var _string = new String("unknown");
    var _command = GLib.spawn_command_line_sync(_path + "/xkblayout-state print \"%E\"",null);
    if (_command[0].toString() == "true")
    {
        _string = _command[1].toString();
    }
    _string = _string.trim().split(/\n/g);

    for (var q = 0; q < _string .length; q++)
    {
		for (var i = 0; i < _codes_table.length; i++)
   	    {
   	    	if(_codes_table[i].split("--")[0] == _string[q].toUpperCase())
    	    {
    	        _string[q] = _codes_table[i].split("--")[1];
    	    }
        }
    }
    return _string;
}

function get_toogle_buttons()
{
    var _string = new String("unknown");
    var _command = GLib.spawn_command_line_sync("setxkbmap -query",null);
    if (_command[0].toString() == "true")
    {
        var _buf = _command[1].toString();
        _string= _buf.substring(_buf.lastIndexOf("grp") + 4, _buf.length );        
    }
    return _string;
}



function _showLayout() 
{
   if(_text != null)
   {
      _text.destroy();
      _layout_timer.stop();
   }
   _text = new St.Label({ style_class: 'show_layout-label', text: _( get_layout() ) });
   global.stage.add_actor(_text);
   _text.set_position(Math.floor (1366 / 2 - _text.width / 2), Math.floor(768 / 2 - _text.height / 2));
   _layout_timer.begin();
}